const Pubsub = require('@google-cloud/pubsub');
const VError = require('verror');
const logger = require('epic-logger');

// const topicName = process.env.TOPIC_NAME;
// const topicName = 'authorization';
// const subscriptionName = process.env.SUBSCRIPTION_NAME;
const subscriptionName = 'authorization';

const pubsub = Pubsub({
  projectId: process.env.GCLOUD_PROJECT,
  keyFilename: process.env.GOOGLE_CLOUD_PUBSUB_KEY,
});

// This configuration will automatically create the topic if
// it doesn't yet exist. Usually, you'll want to make sure
// that a least one subscription exists on the topic before
// publishing anything to it as topics without subscribers
// will essentially drop any messages.
// [START topic]
function getTopic(topicName) { 
  return pubsub.createTopic(topicName)
    .then(data => {
      const topic = data[0]
      return topic
    })
    .catch(error => {
      return pubsub.topic(topicName)
    })
}
// [END topic]

// Used by the worker to listen to pubsub messages.
// When more than one worker is running they will all share the same
// subscription, which means that pub/sub will evenly distribute messages
// to each worker.
// [START subscribe]

/** options => [ 
      { topicName: 'authorization', subscriptionName: 'authorization'}
      { topicName: 'slack', subscriptionName: 'authorization'}
    ]
**/
function subscribe(options, cb) {
  let subscriptions = [];

  // Event handlers
  function handleMessage(subscription) {
    return (message) => cb(null, message, subscription);
  }
  function handleError(err) {
    logger.error(new VError(err), 'failed to handle pubsub message');
    cb(err);
  }

  const topicPromises = options.map(async option => {
    return await getTopic(option.topicName)
  });

  Promise.all(topicPromises)
    .then(topics => {     
      topics.forEach((topic, index) => {
        const option = options[index];
        const fullName = `${option.topicName}_${option.subscriptionName}`;

        topic.createSubscription(fullName)
          .then(subscriptions => {
            const subscription = subscriptions[0]

            subscription.on('message', handleMessage(subscription));
            subscription.on('error', handleError)
            logger.info(
              `Listening to ${option.topicName} with subscription ${
                option.topicName
              }_${option.subscriptionName}`
            )
          })
      });
    })
    .catch(cb);
  // Subscription cancellation function
  return () => {
    if (subscriptions.length > 0) {
      subscriptions.forEach(subscription => {
        // Remove event listeners
        subscription.removeListener('message', handleMessage);
        subscription.removeListener('error', handleError);
        subscription = undefined;
      });
    }
  };
}
// [END subscribe]

module.exports = {
  subscribe,
  getTopic,
};
