const producer = require('./producer');
const background = require('./background');
const VError = require('verror');
const logger = require('epic-logger');

const podName = process.env.POD_NAME || 'no-pod-name';

function subscribe(options, handlers, serviceName = '', payLoadEmit = false) {
  // Subscribe to Cloud Pub/Sub and receive messages to process.
  // The subscription will continue to listen for messages until the process
  // is killed.
  // [START subscribe]
  // [END subscribe]
  return background.subscribe(options, (err, message, subscription) => {
    // Any errors received are considered fatal.
    if (err) {
      logger.error(new VError(err), 'failed to subscribe to topic');
      throw err;
    }

    const isCronMessage =
      message.attributes.type && message.attributes.type === 'cron';

    const payload = isCronMessage ? {} : JSON.parse(message.data.toString());

    let handler = handlers[message.attributes.eventType];

    if (handler && process.env.NODE_ENV !== 'test') {
      payload.updatedAt = message.publishTime || message.attributes.timestamp;

      if (typeof handler !== 'function') {
        subscription.modifyAckDeadline_(
          message.ackId,
          handler.timeoutInSeconds,
          message.connectionId
        );
        handler = handler.method;
      }

      handler(payload, message.attributes)
        .then(result => {
          logger.info(
            `Successfully handled payload ${JSON.stringify(
              payload
            )} for event: ${JSON.stringify(message.attributes)}`
          );

          message.ack();

          if (serviceName.length !== 0 && payLoadEmit) {
            const publishMessageWithPayLoad = {
              data: {
                actionType: 'subscribe',
                eventTopic: subscription.name
                  .split('/subscriptions/')[1]
                  .split('_')[0],
                emittedEventType: message.attributes.eventType,
                serviceName,
                payload
              },
              attributes: { eventType: 'EventPayloadSync' }
            };
            producer.emit(publishMessageWithPayLoad, 'pubsub-registry');
          }
        })
        .catch(error => {
          logger.error(
            `Failed to handle payload: ${JSON.stringify(
              payload
            )} for event: ${JSON.stringify(message.attributes)} with error `,
            error
          );
        });
    } else {
      message.ack();
    }
  });
}

module.exports = {
  subscribe,
  ...producer,
  getTopic: background.getTopic
};
