process.env.GCLOUD_PROJECT = 'microservices-kube';
process.env.PUBSUB_EMULATOR_HOST = 'localhost:8085';
const pubsub = require('./index');

describe('Pubsub', () => {
  test(
    'should be able to subscribe to an event',
    done => {
      const data = {
        name: 'platform team',
        email: 'platform.team@andela.com'
      };

      const handlersMap = {
        MessageCreated: async (payload, metadata) => {
          expect(payload.name).toEqual(data.name);
          expect(payload.email).toEqual(data.email);
          done();
        }
      };

      pubsub.subscribe(
        [{ topicName: 'pubsuba', subscriptionName: 'library' }],
        handlersMap
      );

      pubsub.emit(
        { data: { ...data }, attributes: { eventType: 'MessageCreated' } },
        'pubsuba'
      );
    },
    100000
  );

  test(
    'should be able to subscribe to an event using object as handler',
    done => {
      const data = {
        name: 'another platform team',
        email: 'another.platform.team@andela.com'
      };

      const handlersMap = {
        MessageCreated: {
          timeoutInSeconds: 30 * 60,
          method: async (payload, metadata) => {
            expect(payload.name).toEqual(data.name);
            expect(payload.email).toEqual(data.email);
            done();
          }
        }
      };
      pubsub.subscribe(
        [{ topicName: 'pubsubb', subscriptionName: 'library' }],
        handlersMap
      );

      pubsub.emit(
        { data: { ...data }, attributes: { eventType: 'MessageCreated' } },
        'pubsubb'
      );
    },
    100000
  );
});
