const fancyID = require('pushid');
const getTopic = require('./background').getTopic;
const VError = require('verror');
const logger = require('epic-logger');

module.exports = {
  emitModel(model, message, topicName, cb) {
    message.data.id = message.data.id || fancyID();
    model
      .build(message.data)
      .validate()
      .then(() => {
        this.emit(message, topicName, cb);
      })
      .catch(err => {
        const metadata = Object.assign(
          { method: 'emitModel' },
          message.attributes
        );
        logger.error(new VError(err, 'Validation error occured'), metadata);
        // code 3 is grpc.status.INVALID_ARGUMENT
        cb({ message: validateErr.message, code: 3 });
      });
  },

  emit({ data, attributes }, topicName) {
    data.id = data.id || fancyID();
    attributes.timestamp = new Date().toISOString();
    delete attributes.permissions;

    return new Promise((resolve, reject) => {
      getTopic(topicName)
        .then(topic => {
          topic
            .publisher()
            .publish(Buffer.from(JSON.stringify(data)), attributes, err => {
              if (err) {
                logger.error(
                  `Failed to publish payload ${JSON.stringify(
                    data
                  )} for ${JSON.stringify(
                    attributes
                  )} with error ${JSON.stringify(err)}`
                );
                return reject(err);
              } else {
                logger.info(
                  `Succesfully published payload ${JSON.stringify(
                    data
                  )} for ${JSON.stringify(attributes)}`
                );
                return resolve();
              }
            });
        })
        .catch(err => {
          return reject(err);
        });
    });
  }
};
